/// IMPORTING THIRD PARTY PACKAGES
import 'package:flutter/material.dart';
import 'package:zwap_utils/zwap_utils.dart';
import 'package:taastrap/taastrap.dart';

/// IMPORTING LOCAL COMPONENTS KIT
import 'package:zwap_design_system/atoms/atoms.dart';
import 'package:zwap_design_system/molecules/molecules.dart';

/// Custom component to display an adding interaction tag
class ZwapInteractionTag extends StatefulWidget{

  /// The interaction title to display inside the interaction tag element
  final String interactionTitle;

  /// The global tags added to display inside the advices
  final List<String> globalTags;

  ZwapInteractionTag({Key? key,
    required this.interactionTitle,
    required this.globalTags
  }): super(key: key);

  _ZwapInteractionTagState createState() => _ZwapInteractionTagState();

}

/// The state for this custom interaction tag
class _ZwapInteractionTagState extends State<ZwapInteractionTag>{

  /// Is active the view to type and add new tag
  bool _isAddingTag = false;

  /// Is showing the suggested elements to display some suggestions
  bool _isShowingSuggested = false;

  /// The elements added
  List<String> _elements = [];

  /// The text editing controller input to handle the input text to add custom tag
  final TextEditingController _textAddingController = TextEditingController();

  _ZwapInteractionTagState(){
    this._textAddingController.addListener(() => this._showSuggested());
  }

  /// It shows the suggested elements inside a list of values
  void _showSuggested(){
    setState(() {
      this._isShowingSuggested = this._textAddingController.text.trim() != "";
    });
  }

  /// It adds tag to the elements list
  void _addTag(String titleElement){
    this._switchView();
    setState(() {
      this._elements.add(titleElement);
    });
  }

  /// It removes tag from the elements list
  void _removeTag(String titleElement){
    setState(() {
      this._elements.remove(titleElement);
    });
  }

  /// It switches from the view to type and add new tag
  void _switchView(){
    setState(() {
      this._isAddingTag = !this._isAddingTag;
    });
  }

  /// It gets the body with added tags and add new tag button
  Widget _getBodyChild(){
    return ResponsiveRow<List<Widget>>(
      customInternalPadding: EdgeInsets.only(right: 10, bottom: 10),
      children: List<Widget>.generate(this._elements.length, ((index) =>
          Column(
            children: [
              ZwapOpenToTag(
                openToTagText: (this._elements.length - 1 > index) ? Utils.translatedText("add_tag_text_button") : this._elements[index],
                tagIcon: (this._elements.length - 1 > index) ? Icons.add : Icons.close,
                isLeft: this._elements.length - 1 > index,
                isClickAble: true,
                callBackClick: (_) => (this._elements.length - 1 == index) ? this._removeTag(this._elements[index]) : this._switchView(),
              ),
              this._isAddingTag ? this._getInputAdd() : Container()
            ],
          )
      )),
    );
  }

  /// It gets the input widget to display the input to add a tag and a suggested list
  Widget _getInputAdd(){
    return Column(
      children: [
        Padding(
          padding: EdgeInsets.symmetric(vertical: 10),
          child: ZwapInput(
            controller: this._textAddingController,
          ),
        ),
        this._isShowingSuggested ? this._getAddingListView() : Container()
      ],
    );
  }

  /// It gets the adding list view to select one of them
  Widget _getAddingListView(){
    List<String> suggestedElements = [];
    suggestedElements.addAll(this._elements);
    suggestedElements.addAll(widget.globalTags);
    return Padding(
      padding: EdgeInsets.symmetric(vertical: 10),
      child: Container(
        height: 40,
        child: ListView.builder(
            itemCount: suggestedElements.length,
            itemBuilder: (BuildContext context, int index){
              String element = suggestedElements[index];
              return InkWell(
                onTap: () => this._addTag(element),
                child: ZwapText(
                  textColor: ZwapColors.neutral700,
                  text: element,
                  zwapTextType: ZwapTextType.body1Regular,
                ),
              );
            }
        ),
      ),
    );
  }

  Widget build(BuildContext context){
    return LayoutBuilder(builder: (builder, context){
      return this._getBodyChild();
    });
  }

}